Unit DE54RU_p;
{$J+}
interface
uses
SysUtils, Math, { Delphi }
Lstruct, Lfunc;

procedure DE54RU(FF :Func_F_DE; FG :Func_F_DE; FR :Func_F_DE; M :Integer; XN :Real;
                var YN :Array of Real; XK :Real; HMIN :Real;
                EPS :Real; P :Real; var H :Real; var Y :Array of Real;
                var RA :Array of Real; var IERR :Integer);

implementation

procedure DE54RU(FF :Func_F_DE; FG :Func_F_DE; FR :Func_F_DE; M :Integer; XN :Real;
                var YN :Array of Real; XK :Real; HMIN :Real;
                EPS :Real; P :Real; var H :Real; var Y :Array of Real;
                var RA :Array of Real; var IERR :Integer);
var
SW,IB1,IB2,J,IJK0,I,IJK1,IJK2 :Integer;
X,XS,Q,H3,R,E,HS,RC,RS,RC2,RS2,RSC,RF,RG,RR,RFSC,R1GSC :Real;
BE,BH,BR,BX :Boolean;
const
ZERO :Real = 0.0;
P5 :Real = 0.5;
OP5 :Real = 1.5;
THREE :Real = 3.0;
FOUR :Real = 4.0;
E5 :Real = 0.5E-4;
label
_1,_2,_3,_4,_5,_6,_7,_8,_9,_10,_11,_12,_13,_14,_15,_16,_17,_18,_19,
_20,_21,_22,_23;
begin
IERR := 0;
for I:=1 to M do
 begin
_1:
  Y[I-1] := YN[I-1];
 end;
if ( XN = XK )
 then goto _23;
E5 := 5.0*EPS;
IB1 := M+M;
IB2 := IB1+M;
BH := True;
BR := True;
BX := True;

H := Sign(Abs(H),XK-XN);
X := XN;
_2:
XS := X;
for J:=1 to M do
 begin
  IJK0 := M+J;
  RA[IJK0-1] := Y[J-1];
_3:
 end;
_4:
HS := H;
Q := X+H-XK;
BE := True;
if ( not ( ((H > ZERO ) and ( Q >= ZERO)
 ) or ( (H < ZERO ) and ( Q <= ZERO)) ) )
 then goto _5;
H := XK-X;
BR := False;
_5:
H3 := H/THREE;

for SW:=1 to 5 do
 begin
  RS := Sin(Y[1-1]);
  RC := Cos(Y[1-1]);
  RSC := RS*RC;
  RS2 := RS*RS;
  RC2 := 1.0-RS2;
  RF := FF(X);
  RG := FG(X);
  RR := FR(X);
  RFSC := RF*RSC;
  R1GSC := RG*RC2;
  RA[0] := RS2-RFSC+R1GSC;
  R1GSC := (1.0-RG)*RSC;
  RA[1] := RR*RC-(RF*RC2-R1GSC)*Y[1];
  if ( M = 2 ) 
   then goto _6;
  RA[2] := ((1.0-RG)*(RC2-RS2)+RFSC+RFSC)*Y[1]-(RF*RS2+
    R1GSC)*Y[2]-RR*RS;
_6:
  for I:=1 to M do
   begin
    Q := H3*RA[I-1];
    IJK0 := M+I;
    IJK1 := IB1+I;
    IJK2 := IB2+I;
    case SW of
     1: goto _7;
     2: goto _8;
     3: goto _9;
     4: goto _10;
     5: goto _11;
    end;
_7:
    R := Q;
    RA[IJK1-1] := Q;
    goto _12;
_8:
    R := P5*(Q+RA[IJK1-1]);
    goto _12;
_9:
    R := THREE*Q;
    RA[IJK2-1] := R;
    R := 0.375*(R+RA[IJK1-1]);
    goto _12;
_10:
    R := RA[IJK1-1]+FOUR*Q;
    RA[IJK1-1] := R;
    R := OP5*(R-RA[IJK2-1]);
    goto _12;
_11:
    R := P5*(Q+RA[IJK1-1]);
    Q := Abs(R+R-OP5*(Q+RA[IJK2-1]));
_12:
    Y[I-1] := RA[IJK0-1]+R;
    if ( SW <> 5 ) 
     then goto _16;

    E := Abs(Y[I-1]);
    R := E5;
    if ( E >= P ) 
     then R := E*E5;

    if ( Q < R ) 
     then goto _15;
    if ( not BX ) 
     then goto _22;
    BR := True;
    BH := False;
    H := P5*H;
    if ( Abs(H) >= HMIN ) 
     then goto _13;

    H := Sign(1.0,H)*HMIN;
    BX := False;
_13:
    for J:=1 to M do
     begin
      IJK0 := M+J;
      Y[J-1] := RA[IJK0-1];
_14:
     end;
    X := XS;
    goto _4;
_15:
    if ( Q >= 0.03125*R ) 
     then BE := False;
_16:
   end;
  case SW of
   1: goto _17;
   2: goto _20;
   3: goto _18;
   4: goto _19;
   5: goto _20;
  end;
_17:
  X := X+H3;
  goto _20;
_18:
  X := X+P5*H3;
  goto _20;
_19:
  X := X+P5*H;
_20:
 end;

if ( not ( (BE ) and ( BH ) and ( BR) ) ) 
 then goto _21;
H := H+H;
BX := True;
_21:
BH := True;
if ( BR ) 
 then goto _2;
H := HS;
goto _23;
_22:
IERR := 65;
_23:
end;

end.
